class CdiscElement:
    def __init__(self, name: str, root: bool = False, **attributes):
        """
        :param attributes: Key 是属性名 , Value是对应的SQL查询返回的列
        :param root: 表明是否是根节点
        :type attributes:
        """
        self.name = name
        self.root = root
        self.attributes = attributes



import abc


class AbstractRegistration(abc.ABC):
    def __init__(self):
        self._registry = None

    @property
    def registry(self): return self._registry

    @registry.setter
    def registry(self, value): self._registry = value



import pathlib

import cjen
from cjen.nene.database_info import DataBaseInfo
from cjen.nene.helper import FileHelper

from pyodm.core.resource.database_resource import DataBaseResource
from pyodm.core.xml.writer.cdisc_xml_writer import CdiscXmlWriter
from pyodm.factory.cdsic_xml_xsd_factory import CdiscXMLXsdFactory
from pyodm.tests.sql_to_odm.mappers.subject_mapper import SubjectMapper
from pyodm.tests.sql_to_odm.otree_reader import OtreeReader
from pyodm.utils.forest import Forest
from pyodm.utils.path_utils import PathUtils


class DataBase(DataBaseResource):

    def __init__(self, database_info, registry):
        super().__init__(database_info)
        self.registry = registry

    def load(self):
        return self.subjects()

    @cjen.operate.mysql.factory(
        clazz=SubjectMapper,
        sql=FileHelper.read(SubjectMapper.sql_file),
        size=-1
    )
    def subjects(self, subject_mappers: list[SubjectMapper] = None, **kwargs):
        return [subject_mapper.mapper(self.registry) for subject_mapper in subject_mappers]

        # return []
        # return [
        #     [dict(SubjectData=dict(SubjectKey=subject_mapper.SubjectKey())),
        #      dict(StudyEventData=dict(StudyEventOID=subject_mapper.StudyEventOID()),
        #           SiteRef=dict(LocationOID=subject_mapper.LocationOID())),
        #      dict(ItemGroupData=dict(ItemGroupOID=subject_mapper.ItemGroupOID(),
        #                              ItemGroupRepeatKey=subject_mapper.ItemGroupRepeatKey())),
        #      dict(ItemData=dict(ItemOID=subject_mapper.ItemOID())),
        #      dict(Value=dict(SeqNum=subject_mapper.SeqNum(), Text=subject_mapper.Text()))
        #      ] for subject_mapper in subject_mappers
        # ]

    def data(self) -> list[list[dict]]:
        return self.subjects()


def xsd_files():
    base = PathUtils.folder("pyodm", __file__).joinpath("model", "v2", "resources", "schema")
    xsds = ["ODM-admindata.xsd", "ODM-clinicaldata.xsd", "ODM-foundation.xsd",
            "ODM-protocol.xsd", "ODM-referencedata.xsd", "ODM-study.xsd"]
    return [base.joinpath(xsd) for xsd in xsds]


if __name__ == '__main__':
    registry = CdiscXMLXsdFactory(data_file=pathlib.Path(""), xsd_files=xsd_files())
    registry.clazz_reader()
    dbc = DataBase(DataBaseInfo.factory(
        dict(host="dev-03.cluster-c9qe4y0vrvda.rds.cn-northwest-1.amazonaws.com.cn", port=3306,
             user='root', pwd='8YTJWOuA7XRK17wRQnw4',
             database='eclinical_edc_dev_846')), registry)
    # dbc.subjects()

    otree = Forest.transform(dbc.data())

    # registry = CdiscSpecificationFactory(data_file=pathlib.Path(""), specification_files=["SpecificationV2.xml"])
    registry.clazz_reader()
    reader = OtreeReader(registry)
    objs = reader.read_data(otree)
    i = 1
    for obj in objs:
        cw = CdiscXmlWriter(obj, pathlib.Path(f"test{i}.xml"))
        cw.write()
        i += 1



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class ConditionDef(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/ConditionDef
    """
    
    OID = Model.Attribute()
    
    Name = Model.Attribute()
    
    CommentOID = Model.Attribute()
    
    
    Description = Model.OneElement()
    
    MethodSignature = Model.ManyElements()
    
    FormalExpression = Model.ManyElements()
    
    Alias = Model.ManyElements()
    



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class Organization(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/Organization+Element
    """
    
    OID = Model.Attribute()
    
    Name = Model.Attribute()
    
    Role = Model.Attribute()
    
    Type = Model.Attribute()
    
    LocationOID = Model.Attribute()
    
    PartOfOrganizationOID = Model.Attribute()
    
    
    Description = Model.OneElement()
    
    Address = Model.ManyElements()
    
    Telecom = Model.ManyElements()
    



import pathlib

from cjen.nene.database_info import DataBaseInfo


from pyodm.core.xml.writer.cdisc_xml_writer import CdiscXmlWriter
from pyodm.factory.cdisc_listable_factory import CdiscListableFactory

from pyodm.tests.test_listable_factory.subject_data_resource import SubjectDataResource
from pyodm.utils.path_utils import PathUtils


def xsd_files():
    base = PathUtils.folder("pyodm", __file__).joinpath("model", "v2", "resources", "schema")
    xsds = ["ODM-admindata.xsd", "ODM-clinicaldata.xsd", "ODM-foundation.xsd",
            "ODM-protocol.xsd", "ODM-referencedata.xsd", "ODM-study.xsd"]
    return [base.joinpath(xsd) for xsd in xsds]


if __name__ == '__main__':
    database_info = DataBaseInfo.factory(
        dict(host="dev-03.cluster-c9qe4y0vrvda.rds.cn-northwest-1.amazonaws.com.cn", port=3306,
             user='root', pwd='8YTJWOuA7XRK17wRQnw4',
             database='eclinical_edc_dev_846'))
    data_source = SubjectDataResource(database_info)
    factory = CdiscListableFactory(data_source, xsd_files())

    odm_data = factory.odm()
    cw = CdiscXmlWriter(odm_data, pathlib.Path(f"result.xml"))
    cw.write()



import abc
import pathlib

from pyodm.factory.cdisc_registry import CdiscRegistry


class AbstractConfigurationReader(abc.ABC):
    """
    读取配置文件的抽象方法定义，load_cdisc_definition()
    """
    @abc.abstractmethod
    def load_cdisc_definition(self, registry: CdiscRegistry, files: list[pathlib.Path]): ...



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class StudyEstimands(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/StudyEstimands
    """
    
    
    StudyEstimand = Model.ManyElements()
    



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class ODM(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/ODM
    """
    
    FileType = Model.Attribute()

    AdminData = Model.ManyElements()

    ClinicalData = Model.ManyElements()

    



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class StudyTiming(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/StudyTiming
    """
    
    OID = Model.Attribute()
    
    Name = Model.Attribute()
    
    
    AbsoluteTimingConstraint = Model.ManyElements()
    
    RelativeTimingConstraint = Model.ManyElements()
    
    TransitionTimingConstraint = Model.ManyElements()
    
    DurationTimingConstraint = Model.ManyElements()
    



import cjen
from cjen import MetaMysql

from pyodm.tests.sql_to_odm.cdisc import CdiscSQL
from pyodm.tests.sql_to_odm.cdisc_element import CdiscElement
from pyodm.utils.path_utils import PathUtils


# @Mapper(map_file="yyyy")
@CdiscSQL(sql_path=PathUtils.folder("pyodm", __file__).joinpath("unit_tests", "sql_to_odm", "sqls", "subjects.sql"),
          mapper_path=PathUtils.folder("pyodm", __file__).joinpath("unit_tests", "sql_to_odm", "mappers", "mapper.xml"))
class SubjectMapper(MetaMysql): ...
# SubjectData = CdiscElement(name="SubjectData", root=True, SubjectKey="SubjectKey")
# StudyEventData = CdiscElement(name="StudyEventData", StudyEventOID="StudyEventOID")
# SiteRef = CdiscElement(name="SiteRef", LocationOID="LocationOID")
# ItemGroupData = CdiscElement(name="ItemGroupData", ItemGroupOID="ItemGroupOID", ItemGroupRepeatKey="ItemGroupRepeatKey")
# ItemData = CdiscElement(name="ItemData", ItemOID="ItemOID")
# Value = CdiscElement(name="Value", SeqNum="SeqNum", Text="Text")

# @cjen.operate.common.value
# def SubjectKey(self): ...
#
# @cjen.operate.common.value
# def StudyEventOID(self): ...
#
# @cjen.operate.common.value
# def ItemGroupOID(self): ...
#
# @cjen.operate.common.value
# def ItemGroupRepeatKey(self): ...
#
# @cjen.operate.common.value
# def ItemOID(self): ...
#
# @cjen.operate.common.value
# def SeqNum(self): ...
#
# @cjen.operate.common.value
# def Text(self): ...
#
# @cjen.operate.common.value
# def LocationOID(self): ...



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class Flag(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/Flag
    """
    
    
    FlagValue = Model.OneElement()
    
    FlagType = Model.OneElement()
    



import pyodm.model.definition as Model
import pyodm.model.meta.cdisc_odm_entity as Meta


class Standard(Meta.CdiscODMEntity):
    """
    https://wiki.cdisc.org/display/ODM2/Standard
    """
    
    OID = Model.Attribute()
    
    Name = Model.Attribute()
    
    Type = Model.Attribute()
    
    PublishingSet = Model.Attribute()
    
    Version = Model.Attribute()
    
    Status = Model.Attribute()
    
    CommentOID = Model.Attribute()
    
    
